/*
	
					The Hacker's Diet

					 Find Interface				
*/

#include "Hackdiet.h"

#ifdef FIND_SUPPORT

#include <SysEvtMgr.h>				// Needed to define EvtSysEventAvail()

//	findQuery  --  Respond to a global find query

void findQuery(FindParamsPtr findParams)
{
	Err err;
	UInt16 pos;
	UInt16 fieldNum = 0;
	UInt16 cardNo = 0;
	UInt16 recordNum;
	Char *header;
	Boolean done;
	MemHandle recordH;
	MemHandle headerH;
	LocalID dbID;
	DmOpenRef dbP;
	RectangleType r;
	DmSearchStateType searchState;
	SystemPreferencesType prf;
	
	//	Unless otherwise advised, there are no more items to be found.
	
	findParams->more = false;
	
	/*	Since we handle privacy in a somewhat eccentric way,
		we check for privacy on a whole-application basis rather
		than record per records.  If the system perferences are set
		to hide private records, we retrieve our own application
		preferences (recall that since we may be running without
		our own global variables we have to do this here, placing
		the preferences on the stack) and check whether the user has
		marked the log database private.  If so, we don't bother to
		seach it at all and bypass even making an entry in the
		find results.  */
		
	PrefGetPreferences(&prf);
	if (prf.hideSecretRecords) {
		struct hdPreferences aprf;
		UInt16 psize = sizeof aprf;
		
		if (PrefGetAppPreferences(CreatorID, 0, &aprf, &psize, true) != noPreferenceFound) {
			if (aprf.privateLogs) {
				return;
			}
		}
	}

	//	Find the application's data file
	
	err = DmGetNextDatabaseByTypeCreator(true, &searchState, 
				MonthDbType, CreatorID, true, &cardNo, &dbID);
	if (err) {
		return;
	}

	//	Open the monthly log database
	
	dbP = DmOpenDatabase(cardNo, dbID, findParams->dbAccesMode);
	if (dbP == NULL) {
		return;
	}

	//	Display the application header title
	
	headerH = DmGetResource(strRsc, FindHeaderString);
	header = MemHandleLock(headerH);
	done = FindDrawHeader(findParams, header);
	MemHandleUnlock(headerH);
	DmReleaseResource(headerH);
	if (done) {
		findParams->more = true;
	} else {
		// Search all the fields; start from the last record searched
		recordNum = findParams->recordNum;		
		while (true) {
			Boolean match = false;
			struct hdMonthlyLog *md;
			UInt32 whereFound;
			
			/*	Because applications can take a long time to finish a find 
				users like to be able to stop the find.  Stop the find 
				if an event is pending. This stops if the user does 
				something with the device.  Because this call slows down 
				the search we perform it every so many records instead of 
				every record.  The response time should still be short 
				without introducing much extra work to the search.  */
			
			if (((recordNum & 7) == 0) &&		// Check for events every eighth month
				EvtSysEventAvail(true)) {
				// Stop the search process.
				findParams->more = true;
				break;
			}
			
			recordH = DmQueryNextInCategory(dbP, &recordNum, dmAllCategories);
				
			//	Have we reached the end of the database?
			
			if (recordH == NULL) {
				break;
			}
			
			md = MemHandleLock(recordH);
			whereFound = hdFindInComments(md, findParams->strToFind, &pos);

			if (whereFound != 0) {
				char monthName[MAX_MONTH_NAME], itemTitle[MAX_MONTH_NAME + 12];
				
				done = FindSaveMatch(findParams, recordNum, pos,
								     fieldNum, whereFound, cardNo, dbID);
				if (done) {
					goto bailout;		//	Oops--no room for this one.  I'll be back....
				}
	
				//	Get the bounds of the region where we will draw the results
				
				FindGetLineBounds(findParams, &r);
				
				//	Display the title of the item (month and year)
				
				stringListItem(MonthNamesStringList, md->month, monthName, sizeof monthName);
				StrPrintF(itemTitle, "%s %d", monthName, md->year);
//StrPrintF(itemTitle, "%8lx", whereFound);
				drawInField(itemTitle, &r);
				findParams->lineNumber++;
			}
bailout:;
			MemHandleUnlock(recordH);
			if (done) {
				break;
			}
			recordNum++;
		}
	}	
	DmCloseDatabase(dbP);	
}

void findGoTo(GoToParamsPtr goToParams, Boolean launchingApp)
{
//	EventType event;
//	UInt16 recordNum = goToParams->recordNum;
	
	/*	If we were already running when the find was initiated, we
		need to close any form which might be modifying the
		database.  Since we look up by year, month, and day, there's
		no need to worry about record numbers changing is the
		case in an application which relies on the record number
		supplied by the go to event.  */
	
	if (!launchingApp) {
		FrmCloseAllForms();
	}
	
	logGotoMonth = hdFindMonth((goToParams->matchCustom >> 16) & 0xFFFF,
							   (goToParams->matchCustom >> 8) & 0xFF);
	showSpecificDay = goToParams->matchCustom & 0xFF;
	FrmGotoForm(MonthlyLogForm);
	
/*	Send an event to select the matching text.
	MemSet (&event, 0, sizeof(EventType));

	event.eType = frmGotoEvent;
	event.data.frmGoto.formID = MonthlyLogForm;
	event.data.frmGoto.recordNum = goToParams->recordNum;
	event.data.frmGoto.matchPos = goToParams->matchPos;
	event.data.frmGoto.matchLen = goToParams->searchStrLen;
	event.data.frmGoto.matchFieldNum = goToParams->matchFieldNum;
	event.data.frmGoto.matchCustom = goToParams->matchCustom;
	EvtAddEventToQueue(&event);
*/
}
#endif