/*
	
					The Hacker's Diet

				 Historical Charting Form
					 
		This file contains all the code for generating and
		displaying historical weight and trend charts.
		
*/

#include "Hackdiet.h"

//	Globals

int historyMonths = 3;					// Months of history to display
UInt32 historyGoTo = 0;					// Date to show as end of history if nonzero			

//	Form globals

static char formTitle[MAX_MONTH_YYYY];	// Form title (static because not copied)
static struct hdMonth *m = NULL;		// Unpacked database for month
static int cYear, cMonth;				// Year and month current in memory
static UInt16 cRecno;						// Current month's record number
static Boolean noPrev, noNext;			// Database extrema flags for scroll buttons
static RectangleType r;					// Plot area rectangle
static short lineHeight, charHgt;		// Line and character height in current font
static UInt32 startDate = 0,				// Chart starting date (seconds)
			 endDate = 0;				// Chart ending date (seconds)
static DateTimeType dtStart, dtEnd,		// Start and end dates as DateTimeType
					dtToday;			// Today's date as DateTimeType
static int historyDuration;				// Duration of current chart
static UInt16 xAxisLength;				// Length of X axis in chart, pixels
static Boolean doubleFit = false;		// Fit trend using floating point ?
static Boolean mslope = false;			// Did we successfully fit a trend ?
static UInt32 lastFitDay = 0;			// Last day rolled into trend fit
static struct iTrendFitContext tFitCtx;	// Trend fitting context for fixed point integer calculations

#define Item(x) FrmGetObjectPtr(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(), (x)))

#define leftMargin 	30					// Left margin for chart
#define rightMargin 12					// Right margin for chart
#define topMargin 2						// Top margin for chart
#define bottomMargin 26					// Bottom margin for chart
#define AxisOffset	3		

#define PlotScaleLine(x1, y1, x2, y2) \
 							r.topLeft.x + leftMargin + (x1),						\
							((r.topLeft.y + r.extent.y) - bottomMargin)	- (y1),		\
							r.topLeft.x + leftMargin + (x2),						\
							((r.topLeft.y + r.extent.y) - bottomMargin)	- (y2)
							
#define PlotLine(x1, y1, x2, y2) WinDrawLine(PlotScaleLine(x1, y1, x2, y2))
							
#define PlotGrayLine(x1, y1, x2, y2) WinDrawGrayLine(PlotScaleLine(x1, y1, x2, y2))
							
#define PlotFilledLine(x1, y1, x2, y2) WinFillLine(PlotScaleLine(x1, y1, x2, y2))

#define SecondsPerDay	(24L * 60 * 60)
#define DaysToSeconds(d) ((d) * SecondsPerDay)
#define SecondsToDays(s) ((s) / SecondsPerDay)

//	changeMonth  --  Prepare to change to a new month

static void changeMonth(void)
{
	if (m != NULL) {
		hdDisposeMonthUnpacked(m);
		m = NULL;
	}
}

/*	needMonth  --  Bring month into memory when needed.  If
				   the requested month is not in the database,
				   m will be NULL upon return from this function.  */

static void needMonth(int year, int month)
{
	if ((year != cYear) || (month != cMonth) || (m == NULL)) {
		changeMonth();
		cYear = year;
		cMonth = month;
		m = hdGetMonthByDate(year, month - 1, &cRecno, false);
	}
}

/*	monthLapse --  Back up (nMonths > 0) or advance (nMonths < 0)
				   a given date (in seconds) by an even number of months.  */

static UInt32 monthLapse(UInt32 edate, int nMonths)
{
	DateTimeType dt;
	UInt16 mdays;
	
	TimSecondsToDateTime(edate, &dt);
	if (nMonths < 0) {
		while (nMonths < 0) {
			if (dt.month == 12) {
				dt.month = 1;
				dt.year++;
			} else {
				dt.month++;
			}
			nMonths++;
		}
	} else {
		while (nMonths > 0) {
			if (dt.month == 1) {
				dt.month = 12;
				dt.year--;
			} else {
				dt.month--;
			}
			nMonths--;
		}
	}
	mdays = DaysInMonth(dt.month, dt.year);
	if (dt.day > mdays) {
		dt.day = mdays;
	}
	return TimDateTimeToSeconds(&dt);
}

/*	determineDatabaseDateRange  --  Obtain the date range present
									in the database, defined as the
									interval from the first day of
									the first monthly log present to the
									last day of the most recent log.  */
									
static void determineDatabaseDateRange(UInt32 *start, UInt32 *end)
{
	struct hdMonth *mq;
	DateTimeType dt;
	
	*start = *end = 0;
	mq = hdGetMonthUnpacked(0);
	if (mq != NULL) {
		dt.hour = dt.minute = dt.second = 0;
		dt.year = mq->year;
		dt.month = mq->month + 1;
		dt.day = 1;
		*start = TimDateTimeToSeconds(&dt);
		hdDisposeMonthUnpacked(mq);
		mq = hdGetMonthUnpacked(hdNumMonths() - 1);
		if (mq != NULL) {
			dt.year = mq->year;
			dt.month = mq->month + 1;
			dt.day = mq->days;
			*end = TimDateTimeToSeconds(&dt);
			hdDisposeMonthUnpacked(mq);
		}
	}
}

/*	getDays  --  Obtain weight, trend, and rung for a given
				 range of days, if available,  When returning
				 data for more than one day, the arithmetic
				 mean of days for which entries are present is
				 returned.  The weight and trend results are
				 expressed in the current displayUnit in the
				 preferences.  */

static void getDays(UInt32 start, int days, weight *w, trend *t, rung *r, int *flagCount)
{
	DateTimeType dt;
	UInt32 wsum, tsum, rsum, wtd, rd;
	double dtsum;
	int i, day;
	UInt16 mdays;
	Boolean overflow = false;
	
	*w = 0;
	*t = 0;
	*r = RUNG_ABSENT;
	*flagCount = 0;
	TimSecondsToDateTime(start, &dt);
	needMonth(dt.year, dt.month);
	mdays = DaysInMonth(dt.month, dt.year);
	day = dt.day - 1;
	wsum = tsum = rsum = wtd = rd = 0;
	for (i = 0; i < days; i++) {
		if (m != NULL) {
			if (m->weightArray[day] != 0) {
				wsum += cvtWeight(m->weightArray[day], (weightunit) m->weightUnit, prefs.displayUnit);
				
				/*	For large values of days, as occur when making complete history charts
					of long term databases, it is possible for the unsigned long tsum to
					overflow.  We detect an incipient overflow when tsum exceeds 3e9 and
					switch to floating point for the balance of the calculation, converting
					back to a trend after computing the average.  This adaptive overflow
					handling permits the vast majority of calls to getDays to work
					entirely in integer arithmetic.  */
					
				if (overflow) {
					dtsum += cvtTrend(m->trendArray[day], (weightunit) m->weightUnit, prefs.displayUnit);
				} else {
					tsum += cvtTrend(m->trendArray[day], (weightunit) m->weightUnit, prefs.displayUnit);
					if (tsum > 3000000000ul) {
						dtsum = tsum;
						overflow = true;
					}
				}
				wtd++;
			}
			if (start > lastFitDay) {
				if (doubleFit) {
					DfitSlopePoint(cvtTrend(m->trendArray[day], (weightunit) m->weightUnit, prefs.displayUnit));
				} else {
					fitSlopePoint(&tFitCtx, cvtTrend(m->trendArray[day], (weightunit) m->weightUnit, prefs.displayUnit));
				}
				mslope = true;
				lastFitDay = start;
			}
			if (m->rungArray[day] != RUNG_ABSENT) {
				rsum += m->rungArray[day];
				rd++;
			}
			if (m->flagsArray[day]) {
				(*flagCount)++;
			}
		}
		day++;
		start += DaysToSeconds(1);
		if (day >= mdays) {
			day = 0;
			dt.month++;
			if (dt.month > 12) {
				dt.month = 1;
				dt.year++;
			}
			needMonth(dt.year, dt.month);
			mdays = DaysInMonth(dt.month, dt.year);
		}
	}
	if (wtd > 0) {
		*w = (weight) (wsum / wtd);
		if (overflow) {
			*t = (trend) (dtsum / wtd);
		} else {
			*t = (trend) (tsum / wtd);
		}
	}
	if (rd > 0) {
		*r = (rung) (rsum / rd);
	}
//#define FAKE_RUNG
#ifdef FAKE_RUNG
	*r = 10 + ((12 * (start - startDate)) / (endDate - startDate));
#endif
}
							
//	plotPlanLine  --  Plot a segment of the diet plan on the chart

static void plotPlanLine(UInt16 sx, UInt16 sy, UInt16 ex, UInt16 ey)
{
#define PLAN_PATTERN_LINE

#ifdef PLAN_GRAY_LINE
	PlotGrayLine(sx, sy, ex, ey);
#endif
#ifdef PLAN_PATTERN_LINE
	static CustomPatternType pat[] = { 0xF0, 0xF0, 0xF0, 0xF0,
									   0xF0, 0xF0, 0xF0, 0xF0 };
	
	WinSetPattern(pat);
	PlotFilledLine(sx, sy, ex, ey);
#endif
}

/*	drawCaption  --  Draw caption at the bottom of the gadget, erasing
					 any prior caption.  */
					 
static void drawCaption(char *caption)
{
	
	Int16 wid = FntCharsWidth(caption, StrLen(caption));
	RectangleType er;
	
	er = r;
	er.topLeft.y = (r.topLeft.y + r.extent.y) - lineHeight;
	er.extent.y = lineHeight;
	WinEraseRectangle(&er, 0);
	WinDrawChars(caption, StrLen(caption), r.topLeft.x + (r.extent.x - wid) / 2,
					(r.topLeft.y + r.extent.y) - lineHeight);
}

//	drawChart  --  Draw the chart gadget

static void drawChart(FormPtr fp, UInt16 gidget)
{
	int i;
	UInt16 gudget = FrmGetObjectIndex(fp, gidget);
	UInt16 pixelsPerDay, daysPerPixel, maxLabelRows;
	static int factors[] = {1, 2, 5};
	int factor = 0, power = 1, vunit = 1;
	weight plotw;
	rung lrung = RUNG_ABSENT;			// Last rung plotted, if any
	weight weightMin, weightMax;		// Weight mininum and maximum
	Boolean plotPlan = false;			// Plot diet plan in chart?
	weight planSwstart, planSwend;		// Plan plot start and end weights
	UInt32 planStartDay, planEndDay, planFlatStartDay; // Plan plot day ranges
	double tslope;						// Slope of fit trend
	UInt32 nDays = SecondsToDays(endDate - startDate) + 1;
	UInt32 nFlagged = 0;					// Number of days flagged
	char caption[80];					// Progress and caption string
	DateTimeType dt;
	UInt32 cd = startDate, ncd;
	char lblStr[6];
	Int16 cw, scw, w;
	int lblinc;
	Boolean single = false;
	Boolean yearStart;
	int flgc;

	FrmGetObjectBounds(fp, gudget, &r);
	WinEraseRectangle(&r, 0);
	lineHeight = FntLineHeight();
	charHgt = FntCharHeight();

	pixelsPerDay = (r.extent.x - (leftMargin + rightMargin)) / nDays;
	daysPerPixel = nDays / (r.extent.x - (leftMargin + rightMargin));
	doubleFit = nDays > 366;
							
	//	Draw axes for chart
		
	PlotLine(-AxisOffset, -AxisOffset, -AxisOffset, r.extent.y - (topMargin + bottomMargin));
	xAxisLength = r.extent.x - (leftMargin + rightMargin);
	PlotLine(-AxisOffset, -AxisOffset, xAxisLength, -AxisOffset);
	
	//	If we're using colour, paint the plot area a pretty colour
	
	if (useColour) {
		RectangleType plotRect;
		
		RctSetRectangle(&plotRect, r.topLeft.x + leftMargin - (AxisOffset - 1),
								   r.topLeft.y + topMargin,
								   xAxisLength + AxisOffset,
								   (r.extent.y - (topMargin + bottomMargin)) + AxisOffset);
		WinPushDrawState();
		WinSetForeColor(PrefColourIndex(chartBackgroundColour));
		WinSetBackColor(PrefColourIndex(chartBackgroundColour));
		WinFillRectangle(&plotRect, 0);
		WinPopDrawState();
	}
	
	//	Make first pass through data to determine weight and trend extrema
	
	hdWeightExtrema(dtStart.year, dtStart.month - 1,
					dtEnd.year, dtEnd.month - 1,
					&weightMin, &weightMax);

	/*	If the user wishes to include the diet plan in the chart,
		determine the plan extrema for the month and include them
		in the extrema determined from the weight and trend arrays.  */		
			
	if (PrefFlag(pfPlotPlan) && (prefs.calcStartDate != 0) &&
		(prefs.calcStartWeight != 0) && (prefs.calcGoalWeight != 0)) {
		double startWeight,				// Plan starting weight
			   goalWeight,				// Plan goal weight
			   swstart, swend;			// Sloping weight extrema within month
		UInt32 mstart, mend,				// First and last days of month as seconds
			  sstart, send,				// Date range of sloping trend for this month
			  planEndDate;				// End of plan date
		weight pmin = WEIGHT_MAX,		// Plan plot weight extrema
			   pmax = 0;

#define weightToDouble(w)	((w) / 100.0)
#define doubleToWeight(d)	((weight) ((d) * 100))

		//	Calculate the end date of the plan
		
		startWeight = weightToDouble(cvtWeight(prefs.calcStartWeight, prefs.weightUnit, prefs.displayUnit));
		goalWeight = weightToDouble(cvtWeight(prefs.calcGoalWeight, prefs.weightUnit, prefs.displayUnit));
		planEndDate = prefs.calcStartDate + ((Int32) ((((goalWeight - startWeight) /
			(((double) prefs.calcCalorieBalance) / CaloriesPerWeightUnit(prefs.displayUnit)))) * 24.0 * 60.0 * 60.0));
		mstart = startDate;
		mend = endDate;
		
		/*	Determine whether the range of dates in the plan contains
			any days of the month we're charting.  If so, then a portion
			of the sloping line from the plan's starting weight to the
			goal will appear on this chart.  Calculate the weight and
			date range of the portion of the plan to be shown on this
			chart.  */
		
		if ((prefs.calcStartDate <= mend) && (planEndDate >= mstart)) {
	
			//	Date range of sloping plan in this month
			
			sstart = max(prefs.calcStartDate, mstart);
			send = min(planEndDate, mend);
			
			//	Weight range of sloping plan in this month
			
			swstart = startWeight + ((goalWeight - startWeight) *
						(((double) (sstart - prefs.calcStartDate)) / (planEndDate - prefs.calcStartDate)));
			swend = startWeight + ((goalWeight - startWeight) *
						(((double) (send - prefs.calcStartDate)) / (planEndDate - prefs.calcStartDate)));
		
			planSwstart = doubleToWeight(swstart);
			planSwend = doubleToWeight(swend);
			pmin = min(pmin, min(planSwstart, planSwend));
			pmax = max(pmax, max(planSwstart, planSwend));
			planStartDay = sstart;
			planEndDay = send;
		} else {
			planStartDay = 0;				// Plan interval does not overlap date range of this chart
		}
		
		/*	Next, we have to consider the case in which the plan ends
			prior to the last date in this chart (which includes the case where
			the plan ends before the start of the chart).  If that
			situation obtains, we need to plot a flat line between the
			end of the plan or the start of the chart, whichever is
			later, to the last day in the chart.  */
			
		if (planEndDate < mend) {
			sstart = max(planEndDate, mstart);
			planFlatStartDay = sstart;
			planSwend = doubleToWeight(goalWeight);
			pmin = min(pmin, planSwend);
			pmax = max(pmax, planSwend);
		} else {
			planFlatStartDay = 0;			// Plan does not end before the end of this month
		}
		
		/*	Roll the extrema of the plan plot into those
			determined from the weight log.  But if there was no
			data in the log, skip plotting the plan.  */

		if (weightMax > 0) {
			weightMax = max(weightMax, pmax);
			weightMin = min(weightMin, pmin);
		}
		
		plotPlan = (planStartDay > 0) || (planFlatStartDay > 0);
#undef weightToDouble
#undef doubleToWeight
	}
	
	//	Determine vertical weight scaling based on extrema
	
	if (weightMax > 0) {

		/*	If there's only one day's worth of data or all the entries
			in the log are identical (it *can* happen), it's possible for
			weightMin and weightMax to be identical, which will result in
			division by zero in the autoscaling and plotting code.  If
			this is the case, we arbitrarily expand the extrema by one
			tenth of a weight unit to plot the data centred on the
			chart.  */
			
		if (weightMin == weightMax) {
			weightMin -= 10;
			weightMax += 10;
		}
		
		maxLabelRows = (r.extent.y - (topMargin + bottomMargin)) / lineHeight;
		
		/*	Find a display scale power and factor which permits
			a suitable number of labels on the weight axis.  We do
			this using units like a vintage Tektronix oscilloscope:
			increasing powers of 1, 2, and 5.  */
			
		factor = 0;
		
		while (((weightMax - (weightMin - (weightMin % vunit))) / (factors[factor] * power)) > maxLabelRows) {
			factor++;
			if (factor > 2) {
				factor = 0;
				power *= 10;
			}
			vunit = factors[factor] * power;
		}
		
		/*	There's no point in using a finer-grained weight unit than
			the one we use to plot weights.  Adjust the weight unit if
			it's smaller than the label increment.  */
			
		if (prefs.displayUnit == wStone) {
			if (vunit < 100) {
				vunit = 100;
			}
		} else {
			if (vunit < 10) {
				vunit = 10;
			}
		}
		
		weightMin -= weightMin % vunit;
	} else {
	
		//	Yaaar!  No logged weights in specified interval
		
		char s[132];
		Int16 wid;
		
		drawCaption("");
		stringResource(NoWeightLogItemsString, s, sizeof s);
		wid = FntCharsWidth(s, StrLen(s));
		WinDrawChars(s, StrLen(s), r.topLeft.x + (r.extent.x - wid) / 2,
						r.topLeft.y + (r.extent.y / 2));
		return;					// We're out of here!
	}

	//	Now let's introduce the macros which map weight into Y axis position
	
#define WeightToY(w) ((UInt16) (((((long) (w)) - weightMin) * (r.extent.y - (bottomMargin + topMargin))) / (weightMax - weightMin)))
#define DisplayWeightToY(w) ((UInt16) (((((long) (w)) - weightMin) * (r.extent.y - (bottomMargin + topMargin))) / (weightMax - weightMin)))
#define TrendToY(t) (WeightToY(TrendToWeight(t)))	
#define DisplayTrendToY(t) (DisplayWeightToY(TrendToWeight(t)))	
#define RungToY(w) ((UInt16) (((((long) (w)) - 1) * (r.extent.y - (bottomMargin + topMargin))) / RUNG_MAX))
		
	//	Label weight axis at the left

//#define DUMPSCALE
#ifdef DUMPSCALE		
{
	char s[80];
	
	StrPrintF(s, "^=%d v=%d, Vu=%d, R=%d",
		weightMin, weightMax, vunit, maxLabelRows);
	WinDrawChars(s, StrLen(s), r.topLeft.x, r.topLeft.y);
}
#endif
	for (plotw = weightMin; plotw <= weightMax; plotw += vunit) {
		Int16 wid;
		char txt[12];
		
		editWeight(txt, plotw);
		wid = FntCharsWidth(txt, StrLen(txt));
		WinDrawChars(txt, StrLen(txt),
			r.topLeft.x + (leftMargin - 4) - wid,
			((r.topLeft.y + r.extent.y) - bottomMargin) - (DisplayWeightToY(plotw) + (charHgt / 2)));
	}
	
	//	Plot weight and trend

	stringResource(PlottingChartCaptionString, caption, sizeof caption);
	drawCaption(caption);
	lastFitDay = 0;
	if (doubleFit) {
		DfitSlopeStart();
	} else {
		fitSlopeStart(&tFitCtx);
	}
	if (pixelsPerDay > 1) {
		UInt32 cdate;
		weight w;
		trend t;
		rung rg, lrg = RUNG_ABSENT;
		int pix, opix;
		Int16 oy = -1;
				
		
		for (cdate = startDate; cdate <= endDate; cdate += DaysToSeconds(1)) {
			pix = (xAxisLength * SecondsToDays(cdate - startDate)) / SecondsToDays(endDate - startDate);
			getDays(cdate, 1, &w, &t, &rg, &flgc);
			nFlagged += flgc;
			if (w != 0) {
				UInt16 ny;
				UInt16 wy;
				int offset;
								
				//	Plot trend
				
				ny = TrendToY(t);
				if (pix > 0) {
					if (useColour) {
						WinPushDrawState();
						WinSetForeColor(PrefColourIndex(chartTrendColour));
					}
					if (oy >= 0) {
						PlotLine(opix, oy, pix, ny);
					}
					if (useColour) {
						WinPopDrawState();
					}
				}
				oy = ny;
				
				//	Plot weight
				
				wy = WeightToY(w);
				offset = ((int) wy) - ((int) ny);
				
				if (offset < -2 || offset > 2) {
					if (useColour) {
						WinPushDrawState();
						WinSetForeColor(PrefColourIndex(chartFlsinkColour));
					}
					PlotLine(pix, ny + ((offset > 0) ? 1 : -1), pix, wy + ((offset > 0) ? -2 : 2));
					if (useColour) {
						WinPopDrawState();
					}
				}
				
				//	If today is Flagged, plot the float or sinker as a solid diamond
				
				if (flgc > 0) {
					if (useColour) {
						WinPushDrawState();
						WinSetForeColor(PrefColourIndex(chartFlagColour));
					}
					PlotLine(pix - 2, wy, pix + 2, wy);
					PlotLine(pix, wy - 2, pix, wy + 2);
					if (useColour) {
						WinPopDrawState();
					}
				}
				
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartBobColour));
				}
				PlotLine(pix - 2, wy, pix, wy - 2);
				PlotLine(pix, wy - 2, pix + 2, wy);
				PlotLine(pix + 2, wy, pix, wy + 2);
				PlotLine(pix, wy + 2, pix - 2, wy);
				if (useColour) {
					WinPopDrawState();
				}
			}
			
			//	Plot exercise rung
			
			if ((lrg != RUNG_ABSENT) && (lrg != 0)) {
				rung rt = lrg;

				lrung = lrg;
				
				if ((rg != RUNG_ABSENT) && (rg != 0)) {
					rt = rg;
				}
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartRungColour));
				}
				PlotGrayLine(opix, RungToY(lrg), pix, RungToY(rt));
				if (useColour) {
					WinPopDrawState();
				}
			}
			lrg = rg;

			opix = pix;
		}
	} else {	//	More than one day per pixel
		weight w;
		trend ot = 0, t;
		rung rg;
		UInt16 oty;

		for (i = 0; i < xAxisLength; i++) {
			UInt32 sDate, eDate;
			UInt32 nd;
			UInt16 nty;
			
			sDate = startDate + DaysToSeconds((SecondsToDays(endDate - startDate) * i) / xAxisLength);
			eDate = startDate + DaysToSeconds((SecondsToDays(endDate - startDate) * (i + 1)) / xAxisLength);
			nd = SecondsToDays(eDate - sDate);
			if (nd == 0) {
				nd = 1;
			}
			getDays(sDate, nd, &w, &t, &rg, &flgc);
			nFlagged += flgc;
			nty = DisplayTrendToY(t);
			if ((ot > 0) && (t > 0)) {
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartTrendColour));
				}
				PlotLine(i - 1, oty, i, nty);
				if (useColour) {
					WinPopDrawState();
				}
			}
#ifdef SHOW_WEIGHT_DOTS
			if (w > 0) {
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartBobColour));
				}
				PlotLine(i, DisplayWeightToY(w), i, DisplayWeightToY(w));
				if (useColour) {
					WinPopDrawState();
				}
			}
#endif
			ot = t;
			oty = nty;
			
			if (rg != RUNG_ABSENT) {
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartRungColour));
				}
				PlotGrayLine(i, RungToY(rg), i, RungToY(rg));
				if (useColour) {
					WinPopDrawState();
				}
				lrung = rg;
			}
		}
	}
	changeMonth();					// Release any month in memory
	
	//	Complete trend fitting calculation for the interval
	
	if (mslope) {
		if (doubleFit) {
			tslope = DfitSlopeEnd();
		} else {
			tslope = fitSlopeEnd(&tFitCtx);
		}
	}
		
	/*	If a diet plan exists and the user wishes it plotted
		in the chart, do so now.  */

#define DateToX(d)	((int) ((SecondsToDays((d) - startDate) * xAxisLength) / SecondsToDays(endDate - startDate)))
		
	if (plotPlan) {
		UInt16 sx, sy, ex, ey;
		
		if (useColour) {
			WinPushDrawState();
			WinSetForeColor(PrefColourIndex(chartPlanColour));
			WinSetBackColor(PrefColourIndex(chartBackgroundColour));
		}
		if (planStartDay > 0) {
			sx = DateToX(planStartDay);
			sy = WeightToY(planSwstart);
			ex = DateToX(planEndDay);
			ey = WeightToY(planSwend);
			plotPlanLine(sx, sy, ex, ey);
		}
		if (planFlatStartDay > 0) {
			sx = DateToX(planFlatStartDay);
			sy = ey = WeightToY(planSwend);
			ex = DateToX(endDate);
			plotPlanLine(sx, sy, ex, ey);
		}
		if (useColour) {
			WinPopDrawState();
		}
	}
	
	//	Label rungs at right side of chart (draw only if any rung present)
	
	if (lrung != RUNG_ABSENT) {
		char lrs[12];
		Int16 wid;
		
		StrPrintF(lrs, "%d", lrung);
		wid = FntCharsWidth(lrs, StrLen(lrs));
		WinDrawChars(lrs, StrLen(lrs),
					 (r.topLeft.x + r.extent.x) - wid,
					 ((r.topLeft.y + r.extent.y) - bottomMargin) - (RungToY(lrung) + (charHgt / 2)));
#define RUNG_EXCLUSION_ZONE	6			// How many rungs to exclude around last rung in monthly log
										// (Should really be calculated from font metrics and window
										//	geometry).
		for (i = 1; i <= 48; i = ((i / 6) * 6) + 6) {
			if (abs(lrung - i) >= RUNG_EXCLUSION_ZONE) {
				StrPrintF(lrs, "%d", i);
				wid = FntCharsWidth(lrs, StrLen(lrs));
				WinDrawChars(lrs, StrLen(lrs),
							 (r.topLeft.x + r.extent.x) - wid,
							 ((r.topLeft.y + r.extent.y) - bottomMargin) - (RungToY(i) + (charHgt / 2)));
			}		
		}
	}
	
	/*	Label date axis at bottom.  The format depends on the interval
		the chart represents.  */

	/*	For intermediate charts, we label month and year
		boundaries, using three-letter month abbreviations
		if they fit and single-letter abbreviations for
		longer-term charts.  */

	cw = FntCharsWidth("Mar ", 4);
	scw = FntCharsWidth("M ", 2);
	lblinc = (((SecondsToDays(endDate - startDate) / 30) * cw) + (xAxisLength - 1)) / xAxisLength;
	if (lblinc < 1) {
		lblinc = 1;
	}
	if (lblinc > 1) {
		lblinc = (((SecondsToDays(endDate - startDate) / 30) * scw) + (xAxisLength - 1)) / xAxisLength;
		if (lblinc < 1) {
			lblinc = 1;
		}
		single = true;
	}
	TimSecondsToDateTime(cd, &dt);
	if (dt.day > 1) {
		dt.day = 1;
		dt.month++;
		if (dt.month == 13) {
			dt.year++;
			dt.month = 1;
		}
	}
	ncd = TimDateTimeToSeconds(&dt);
	if (DateToX(ncd) < (single ? scw : cw)) {
		cd = ncd;
	}
	if (lblinc < 4) {
		while (cd <= endDate) {
			FontID fid;
			
			TimSecondsToDateTime(cd, &dt);
			
			/*	For the month of January, show the last two digits of the
				year in bold instead of the month name.  */
			
			yearStart = dt.month == 1;
			if (yearStart) {
				//	Gnarly format required since "%02d" doesn't work on pre-3.0 ROMs
				StrPrintF(lblStr, "%d%d", (dt.year / 10) % 10, dt.year % 10);
				fid = FntSetFont(boldFont);
			} else {
				stringListItem(MonthAbbreviationsStringList, dt.month - 1, lblStr, sizeof lblStr);
				if (single) {
					lblStr[1] = 0;
				}
			}
			w = FntCharsWidth(lblStr, StrLen(lblStr));
			WinDrawChars(lblStr, StrLen(lblStr),
				(r.topLeft.x + leftMargin + DateToX(cd)) - (w / 2),
				((r.topLeft.y + r.extent.y) - bottomMargin) + 4);
			if (yearStart) {
				FntSetFont(fid);
			}
			if (historyDuration == 1) {
				int d, dim = DaysInMonth(dt.month, dt.year) - 7;
				UInt32 dd = cd + DaysToSeconds(7);
				
				for (d = dt.day + 7; (d < dim) && (dd < endDate); d += 7) {
					StrPrintF(lblStr, "%d", d);
					w = FntCharsWidth(lblStr, StrLen(lblStr));
					WinDrawChars(lblStr, StrLen(lblStr),
						(r.topLeft.x + leftMargin + DateToX(dd)) - (w / 2),
						((r.topLeft.y + r.extent.y) - bottomMargin) + 4);
					dd += DaysToSeconds(7);
				}	
			}
			dt.day = 1;
			dt.month++;
			if (dt.month == 13) {
				dt.year++;
				dt.month = 1;
			}
			cd = TimDateTimeToSeconds(&dt);
		}
		
	} else {
	
		/*	For really long-term charts, we only label year boundaries
			in increments of as many years are are required to keep
			adjacent years from "stepping" on one another.  */

		cw = FntCharsWidth("00 ", 3);
		lblinc = (((SecondsToDays(endDate - startDate) / 365) * cw) + (xAxisLength - 1)) / xAxisLength;
		if (lblinc < 1) {
			lblinc = 1;
		}
		while (cd <= endDate) {
			TimSecondsToDateTime(cd, &dt);
			//	Gnarly format required since "%02d" doesn't work on pre-3.0 ROMs
			StrPrintF(lblStr, "%d%d", (dt.year / 10) % 10, dt.year % 10);
			w = FntCharsWidth(lblStr, StrLen(lblStr));
			WinDrawChars(lblStr, StrLen(lblStr),
				(r.topLeft.x + leftMargin + DateToX(cd)) - (w / 2),
				((r.topLeft.y + r.extent.y) - bottomMargin) + 4);
			dt.month = 1;
			dt.day = 1;
			dt.year += lblinc;
			cd = TimDateTimeToSeconds(&dt);
		}
		
	}

	//	Calculate the weekly trend change and calorie balance and add as caption

	{
		signed long weeklyTrendChange, whole, frac;
		char fmt[60], deltaw[12], wunits[16], eunits[12],
			 deltacal[12], flagpercent[6];
		char *sgn;
		long dcal;
		weightunit dwu = prefs.displayUnit;

		if (dwu == wStone) {
			dwu = wPound;
		}

		if (mslope) {
			dcal = (long) (EnergyPerWeightUnit(prefs.displayUnit) * tslope);
			weeklyTrendChange = (signed long) (tslope * 1000000 * 7);
	
			sgn = (weeklyTrendChange < 0) ? "-" : "+";
			if (weeklyTrendChange < 0) {
				weeklyTrendChange = -weeklyTrendChange;
			}
			whole = weeklyTrendChange  / 1000000L;
			frac = ((weeklyTrendChange + 5000L) / 10000L) % 100;
			if (whole == 0 && frac == 0) {
				sgn = "";
			}
			editDeltaWeight(deltaw, sgn, whole, frac);
			NumOut(deltaw);
			stringListItem(WeightUnitAbbreviationsStringList, dwu, wunits, sizeof wunits);
			StrPrintF(deltacal, "%s%ld", (dcal == 0) ? "" : (dcal > 0 ? "+" : "-"), abs(dcal));
			stringListItem(EnergyUnitAbbreviationsStringList, PrefFlag(pfEnergyJoule) ? 1 : 0,
							eunits, sizeof eunits);
			if (nFlagged != 0) {
				StrPrintF(flagpercent, " %d%%", (int) ((nFlagged * 100) / nDays));
			} else {
				flagpercent[0] = 0;
			}
			stringResource(ChartBalanceFormatString, fmt, sizeof fmt);
			StrPrintF(caption, fmt, deltaw, wunits, deltacal, eunits, flagpercent);
			drawCaption(caption);
		} else {
			drawCaption("");
		}
	}
}

/*	tapChart  --  Handle tap in chart.  We determine the corresponding
				  month and day and (if the month exists in the database),
				  display its log, scrolled to the closest day.  */

static void tapChart(FormPtr f, UInt16 gindex, RectanglePtr r, int x, int y)
{
	UInt32 dday = SecondsToDays(startDate) +
		((SecondsToDays(endDate - startDate) * ((x - leftMargin) - r->topLeft.x)) / xAxisLength);
	DateTimeType dt;
	
	TimSecondsToDateTime(DaysToSeconds(dday), &dt);
	if (hdIsMonthPresent(dt.year, dt.month - 1)) {
		logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
		showSpecificDay = dt.day;
		FrmGotoForm(MonthlyLogForm);
	}
}

/*	chartingHandleEvent  --  Handle event while the charting form
							 is displayed.  */
							  
static Boolean chartingHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
	  	
  		case frmOpenEvent:	
			FrmDrawForm(FrmGetActiveForm());
			
			/*	If the duration is one of the standard values, set the
				corresponding duration push button to indicate it.  */
				
			CtlSetValue(Item(HistoricalChartMonthPushButton), historyDuration == 1);
			CtlSetValue(Item(HistoricalChartQuarterPushButton), historyDuration == 3);
			CtlSetValue(Item(HistoricalChartSixMonthsPushButton), historyDuration == 6);
			CtlSetValue(Item(HistoricalChartYearPushButton), historyDuration == 12);
			
			/*	If the duration is standard, disable the button which
				corresponds to it.  */
				
			if ((historyDuration == 1) || (historyDuration == 3) || (historyDuration == 6) ||
				(historyDuration == 12)) {
				CtlSetEnabled(Item(historyDuration == 1 ? HistoricalChartMonthPushButton :
					(historyDuration == 3 ? HistoricalChartQuarterPushButton :
					(historyDuration == 6 ? HistoricalChartSixMonthsPushButton :
					HistoricalChartYearPushButton))), false);
			}
			drawChart(FrmGetActiveForm(), HistoricalChartChartGadget);
			handled = true;
			break;
			
		case frmUpdateEvent:
			FrmDrawForm(FrmGetActiveForm());
			drawChart(FrmGetActiveForm(), HistoricalChartChartGadget);
			handled = true;
			break;
			
		case frmCloseEvent:
			changeMonth();
			MenuEraseStatus(NULL);
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case ctlRepeatEvent:
			switch(event->data.ctlRepeat.controlID) {
				case HistoricalChartPreviousIntervalRepeating:	// Previous interval
prevChart:;			endDate = startDate - DaysToSeconds(1);
					startDate = monthLapse(endDate, historyDuration);
changeChart:		historyMonths = -2;
					FrmGotoForm(HistoricalChartForm);
					handled = true;
					break;
					
				case HistoricalChartNextIntervalRepeating:	// Next interval
nextChart:;			startDate = endDate + DaysToSeconds(1);
					endDate = monthLapse(startDate, -historyDuration);
					goto changeChart;
			}
			break;
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case HistoricalChartYearButton:				// Year button: Show year in which historical chart ends
					yearGotoYear = dtEnd.year;
					FrmGotoForm(YearViewForm);
					handled = true;
					break;
										
				case HistoricalChartTodayButton:			// Today button: Show log entry for today
					{
						DateTimeType dt;
	
						// Mustn't use dtToday since date may have changed
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
					
				case HistoricalChartTrendButton:		// Trend button: Show trend analysis
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
					
				case HistoricalChartMonthPushButton:
					historyDuration = 1;
					goto reChart;
							
				case HistoricalChartQuarterPushButton:
					historyDuration = 3;
					goto reChart;
					
				case HistoricalChartSixMonthsPushButton:
					historyDuration = 6;
					goto reChart;
					
				case HistoricalChartYearPushButton:
					historyDuration = 12;
					goto reChart;
			}
			break;
			
		case penDownEvent:
			{
				FormPtr f = FrmGetActiveForm();
				UInt16 g = FrmGetObjectIndex(f, HistoricalChartChartGadget);
				RectangleType r;
				
				FrmGetObjectBounds(f, g, &r);
				if (RctPtInRectangle(event->screenX, event->screenY, &r)) {
					tapChart(f, g, &r, event->screenX, event->screenY);
					handled = true;
				}
			}
			break;
		
		case keyDownEvent:		
			switch (event->data.keyDown.chr) {
					
				case pageUpChr:
					if (!noPrev) {
						goto prevChart;
					}
					handled = true;				// Or, more precisely, ignored
					break;
					
				case pageDownChr:
					if (!noNext) {
						goto nextChart;
					}
					handled = true;
					break;
			}
			break;

		case menuEvent:
			{
				switch (event->data.menu.itemID) {
						case EditHistoricalChartPreferences:
							FrmGotoForm(PreferencesForm);
							handled = true;
							break;
							
						case ChartHistoricalChartMonth:
							historyDuration = 1;
reChart:;					startDate = monthLapse(endDate, historyDuration);
reChartDraw:;				historyMonths = -2;
							FrmGotoForm(HistoricalChartForm);
							handled = true;
							break;
							
						case ChartHistoricalChartQuarter:
							historyDuration = 3;
							goto reChart;
							
						case ChartHistoricalChartSixMonths:
							historyDuration = 6;
							goto reChart;
							
						case ChartHistoricalChartYear:
							historyDuration = 12;
							goto reChart;
							
						case ChartHistoricalChartEntireHistory:
							historyDuration = 0;
							{
								UInt32 sd, ed;
								
								determineDatabaseDateRange(&sd, &ed);
								if (ed != 0) {
									startDate = sd;
									endDate = ed;
									goto reChartDraw;
								}
							}
							handled = true;
							break;
																					
						case ViewHistoricalChartMonthlyCharts:
							/*	Show monthly chart of last month in historical
								chart unless it's past the end of the database,
								in which case the last month in the database
								is charted.  */
							if (hdIsMonthPresent(dtStart.year, dtStart.month - 1)) {
								logGotoMonth = hdFindMonth(dtStart.year, dtStart.month - 1);
							} else {
								logGotoMonth = 0;
							}
							FrmGotoForm(MonthlyChartForm);
							handled = true;
							break;
							
						case ViewHistoricalChartTrendAnalysis:
							FrmGotoForm(TrendAnalysisForm);
							handled = true;
							break;
							
						case ViewHistoricalChartThisYear:
							yearGotoYear = dtEnd.year;
							FrmGotoForm(YearViewForm);
							handled = true;
							break;						
						
						case ViewHistoricalChartAllYears:
							FrmGotoForm(YearListForm);
							handled = true;
							break;
							
						case ViewHistoricalChartAllMonths:
							FrmGotoForm(MonthsForm);
							handled = true;
							break;
							
						case ViewHistoricalChartExerciseLadder:
							FrmGotoForm(ExerciseForm);
							handled = true;
							break;
							
						case ViewHistoricalChartDietCalculator:
							FrmGotoForm(DietCalcForm);
							handled = true;
							break;
							
						case ViewHistoricalChartKilograms:
							prefs.displayUnit = wKilogram;
							goto reChart;
							
						case ViewHistoricalChartPounds:
							prefs.displayUnit = wPound;
							goto reChart;
							
						case ViewHistoricalChartStones:
							prefs.displayUnit = wStone;
							goto reChart;
					
						case ViewHistoricalChartAboutEatWatch:
							aboutDialogue();
							break;
				}	
			}
			handled = true;
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	historyLoad  --  Initialise chart form. */

void historyLoad(FormPtr f)
{
	char monthNameStart[MAX_MONTH_NAME], monthNameEnd[MAX_MONTH_NAME];
	UInt32 firstDate, lastDate, todaysDate;

	m = NULL;
	cYear = cMonth = -1;
	
	todaysDate = TimGetSeconds();
	TimSecondsToDateTime(todaysDate, &dtToday);
	determineDatabaseDateRange(&firstDate, &lastDate);
			
	/*	If a history of a specific length has been requested, determine
		the corresponding range of dates.  */

	if (historyMonths != -2) {
		if ((historyMonths != -1) || (startDate == 0) || (endDate == 0) || (endDate <= startDate)) {
			if (historyMonths == -1) {
				historyMonths = 3;
			}
			if (historyMonths == 0) {
				startDate = firstDate;
				endDate = lastDate;
			} else {
				if (historyGoTo != 0) {
					endDate = min(historyGoTo, todaysDate);
				} else {
					endDate = todaysDate;
				}
				startDate = monthLapse(endDate, historyMonths);
			}
			historyDuration = historyMonths;		// Save current chart duration
		}
	}
	
	/*	Just one more thing.  If the end date is in the future, adjust it to
		today's days, and back up the start date accordingly.  */
		
	if (endDate > todaysDate) {
		startDate -= (endDate - todaysDate);
		if (startDate < firstDate) {
			startDate = firstDate;
		}
		endDate = todaysDate;
	}
	historyMonths = -1;
	historyGoTo = 0;
	
	/*	Now that we've definitively established the start and and dates, publish
		for other interested parties in DateTimeType format.  */

	TimSecondsToDateTime(startDate, &dtStart);
	TimSecondsToDateTime(endDate, &dtEnd);
	
	//	Hide next and/or previous repeating buttons if we're at the end of the database
	
	noNext = noPrev = false;
	if (startDate <= firstDate) {
		CtlHideControl(FrmGetObjectPtr(f, FrmGetObjectIndex(f, HistoricalChartPreviousIntervalRepeating)));
		noPrev = true;
	}
	if ((endDate >= lastDate) || (endDate >= todaysDate) ||
		((dtToday.year == dtEnd.year) && (dtToday.month == dtEnd.month))
	   ) {
		CtlHideControl(FrmGetObjectPtr(f, FrmGetObjectIndex(f, HistoricalChartNextIntervalRepeating)));
		noNext = true;
	}
	
	FrmSetEventHandler(f, chartingHandleEvent);
	
	//	Set the form title to the date range, depending on its extent
	
	if (dtStart.year == dtEnd.year) {
		if (dtStart.month == dtEnd.month) {
			stringListItem(MonthNamesStringList, dtStart.month - 1, monthNameStart, sizeof monthNameStart);
			StrPrintF(formTitle, "%s %d", monthNameStart, dtStart.year);
		} else {
			stringListItem(MonthAbbreviationsStringList, dtStart.month - 1, monthNameStart, sizeof monthNameStart);
			stringListItem(MonthAbbreviationsStringList, dtEnd.month - 1, monthNameEnd, sizeof monthNameEnd);
			StrPrintF(formTitle, "%s - %s %d", monthNameStart, monthNameEnd, dtStart.year);
		}
	} else {
		stringListItem(MonthAbbreviationsStringList, dtStart.month - 1, monthNameStart, sizeof monthNameStart);
		stringListItem(MonthAbbreviationsStringList, dtEnd.month - 1, monthNameEnd, sizeof monthNameEnd);
		StrPrintF(formTitle, "%s %d - %s %d", monthNameStart, dtStart.year, monthNameEnd, dtEnd.year);
	}
	FrmSetTitle(f, formTitle);
}