/*
	
					The Hacker's Diet

					 Year View Form
		
*/

#include "Hackdiet.h"			

//	Application globals

int yearGotoYear = -1;					// Which year to show (current year if -1)

//	Form globals

static Boolean noPrev, noNext;			// Database extrema flags for scroll buttons
static int prevYear, nextYear;			// Previous and next years if Boolean above is true
static int currentYear = -1;			// Current year we're showing

static Boolean presentInDB[12];			// Is this month present in the database ?

#define leftMargin 		0
#define bottomMargin	0

//	drawCalendar  --  Draw the calendar gadget

static void drawCalendar(FormPtr fp, UInt16 gidget)
{
	int i, j, width, height, bold = -1;
	RectangleType r, cr;
	UInt16 gudget = FrmGetObjectIndex(fp, gidget);
	short lineHeight = FntLineHeight(), charHgt = FntCharHeight();
	DateTimeType today;

	TimSecondsToDateTime(TimGetSeconds(), &today);
	
	FrmGetObjectBounds(fp, gudget, &r);
	width = r.extent.x;
	height = r.extent.y;
	
	WinEraseRectangle(&r, 0);
	WinDrawRectangleFrame(rectangleFrame, &r);
		
#define PlotLine(x1, y1, x2, y2) \
 				WinDrawLine(r.topLeft.x + leftMargin + (x1),						\
							((r.topLeft.y + r.extent.y - 1) - bottomMargin)	- (y1),		\
							r.topLeft.x + leftMargin + (x2),						\
							((r.topLeft.y + r.extent.y - 1) - bottomMargin)	- (y2))
							
#define PlotGrayLine(x1, y1, x2, y2) \
 				WinDrawGrayLine(r.topLeft.x + leftMargin + (x1),					\
							((r.topLeft.y + r.extent.y - 1) - bottomMargin) - (y1),		\
							r.topLeft.x + leftMargin + (x2),						\
							((r.topLeft.y + r.extent.y - 1) - bottomMargin) - (y2))
	
	//	Horizontal gridlines for each quarter
	
	for (i = 1; i < 4; i++) {
		PlotLine(0, (height / 4) * i, width, (height / 4) * i);
	}
	
	//	Vertical gridlines between months
	
	for (i = 1; i < 3; i++) {
		PlotLine((width / 3) * i, 0, (width / 3) * i, height);
	}
	
	//	Paint month abbreviations
	
	for (i = 0; i < 4; i++) {
		int y = ((height / 4) * i) + ((height / 8) - (charHgt / 2));
		
		for (j = 0; j < 3; j++) {
			int x = (width / 3) * j, n = (i * 3) + j, df;
			char mabbr[12];
			
			RctSetRectangle(&cr, r.topLeft.x + leftMargin + x, r.topLeft.y + y,
							width / 3, height / 4);
			df = presentInDB[n];
			if (df != bold) {
				FntSetFont(df ? boldFont : stdFont);
				bold = df;
			}
			stringListItem(MonthAbbreviationsStringList, (i * 3) + j, mabbr, sizeof mabbr);
			drawInFieldCentre(mabbr, &cr);
			
			if ((today.year == currentYear) && ((today.month - 1) == n)) {
				cr.topLeft.x += 3;
				cr.topLeft.y -= ((height / 8) - (charHgt / 2)) - 3;
				cr.extent.x -= 6;
				cr.extent.y -= 6;
				WinInvertRectangle(&cr, 0);
			}
		}
	}
	if (bold) {
		FntSetFont(stdFont);
	}						
}

/*	askCreateNewMonth  --  Ask user if we should create a new blank monthly
						   log for the given month.  */
						   
static Boolean askCreateNewMonth(UInt16 year, UInt16 month)
{
	char mname[MAX_MONTH_NAME], yname[8];
	
	stringListItem(MonthNamesStringList, month, mname, sizeof mname);
	StrIToA(yname, year);
	return FrmCustomAlert(CreateNewMonthLogAlert, mname, yname, NULL) == 0;
}

/*	tapCalendar  --  Handle tap in calendar.  If the month tapped is
					 present in the database, display its monthly log.
					 If the month is not present, ask the user whether
					 a blank log for the given month should be added
					 and if so, create and display it.  We do not permit
					 creation of logs for months subsequent to the current
					 date.  */

static void tapCalendar(FormPtr f, UInt16 gindex, RectanglePtr r, int x, int y)
{
	int cx = (x - r->topLeft.x) / (r->extent.x / 3),
		cy = (y - r->topLeft.y) / (r->extent.y / 4),
		n;
		
	n = (cy * 3) + cx;
	if (presentInDB[n]) {
		logGotoMonth = hdFindMonth(currentYear, n);
		FrmGotoForm(MonthlyLogForm);
	} else {
		DateTimeType dt;

		TimSecondsToDateTime(TimGetSeconds(), &dt);
		if ((currentYear < dt.year) || (n <= (dt.month - 1))) {
			if (askCreateNewMonth(currentYear, n)) {
				hdEnsureMonthPresent(currentYear, n);
				logGotoMonth = hdFindMonth(currentYear, n);
				FrmGotoForm(MonthlyLogForm);
			} else {
				//	Go to current year to force repaint after alert is dismissed
				yearGotoYear = currentYear;
				FrmGotoForm(YearViewForm);
			}
		}
	}
}

/*	yearViewHandleEvent  --  Handle event while the year view form
							 is displayed.  */
							  

static Boolean yearViewHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
	  	
  		case frmOpenEvent:	
			FrmDrawForm(FrmGetActiveForm());
			drawCalendar(FrmGetActiveForm(), YearViewCalendarGadget);
			handled = true;
			break;
			
		case frmCloseEvent:
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case ctlRepeatEvent:
			switch(event->data.ctlRepeat.controlID) {
				case YearViewPreviousYearRepeating:		// Previous year
					yearGotoYear = prevYear;
changeYear:			FrmGotoForm(YearViewForm);
					handled = true;
					break;
					
				case YearViewNextYearRepeating:			// Next year
					yearGotoYear = nextYear;
					goto changeYear;
			}
			break;
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case YearViewListButton:			// List button: Back to year list
					yearGotoYear = currentYear;
					FrmGotoForm(YearListForm);
					handled = true;
					break;
										
				case YearViewTodayButton:			// Today button: Show log entry for today
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
			}
			break;
			
		case penDownEvent:
			{
				FormPtr f = FrmGetActiveForm();
				UInt16 g = FrmGetObjectIndex(f, YearViewCalendarGadget);
				RectangleType r;
				
				FrmGetObjectBounds(f, g, &r);
				if (RctPtInRectangle(event->screenX, event->screenY, &r)) {
					tapCalendar(f, g, &r, event->screenX, event->screenY);
					handled = true;
				}
			}
			break;
		
		case keyDownEvent:		
			switch (event->data.keyDown.chr) {
					
				case pageUpChr:
					if (!noPrev) {
						yearGotoYear = prevYear;
						goto changeYear;
					}
					handled = true;				// Or, more precisely, ignored
					break;
					
				case pageDownChr:
					if (!noNext) {
						yearGotoYear = nextYear;
						goto changeYear;
					}
					handled = true;
					break;
			}
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	yearViewLoad  --  Initialise year view form. */

void yearViewLoad(FormPtr f)
{
	static char yearNumber[12];
#ifdef OLDWAY
	ControlPtr lp;
#endif

	currentYear = yearGotoYear;
	if (currentYear < 0) {
		DateTimeType dt;

		TimSecondsToDateTime(TimGetSeconds(), &dt);
		currentYear = dt.year;
	}
	yearGotoYear = -1;

	hdMonthsPresent(currentYear, presentInDB, &prevYear, &nextYear);
	noPrev = prevYear < 0;
	noNext = nextYear < 0;
	
	prevYear = currentYear - 1;
	nextYear = currentYear + 1;
	
	//	Hide next and/or previous repeating buttons if we're at the end of the database

	if (noNext) {
		DateTimeType dt;

		TimSecondsToDateTime(TimGetSeconds(), &dt);
		/*	Only allow user to scroll to a year not in the database
			if we're viewing a year prior to today's date.  */
		if (currentYear >= dt.year) {
			CtlHideControl(FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearViewNextYearRepeating)));
		}
	}

	StrPrintF(yearNumber, "%d", currentYear);
#ifdef OLDWAY
	lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, YearViewWhichYearLabel));
	CtlSetLabel(lp, yearNumber);
#else
	FrmCopyLabel(f, YearViewWhichYearLabel, yearNumber);
#endif
	FrmSetEventHandler(f, yearViewHandleEvent);
}